import axios from 'axios'
import _ from 'lodash'
import Vue from 'vue'
/* global msCrypto */

// Copied from: https://stackoverflow.com/a/2117523
const uuidv4 = () => {
  return ([1e7]+-1e3+-4e3+-8e3+-1e11).replace(/[018]/g, c =>
    (c ^ (crypto || msCrypto).getRandomValues(new Uint8Array(1))[0] & 15 >> c / 4).toString(16)
  )
}

const getItemIdxById = (items, id) => {
  return _.findIndex(items, item => item.id === id);
}

const getItemById = (items, id) => {
  return _.find(items, item => item.id === id);
}

const normalizeItem = (item, includeId=true) => {
  // Special attributes for our documentviews
  const ignoreCols = ['attachment_names', 'attachments', 'updated_at'];
  if (!includeId) {
    ignoreCols.push('id');
  }
  return _.omit(item, ignoreCols);
}

const createPayload = (item, entitySingular, includeId=true) => {
  const normalizedItem = normalizeItem(item, includeId);
  const payload = {[entitySingular]: normalizedItem};
  if (item.attachments && item.attachments.length > 0) {
    const formData = new FormData();
    formData.append('json', JSON.stringify(payload));
    _.forEach(item.attachments, a => formData.append('attachments', a.file, a.name || a.file.name));

    return formData;
  } else {
    return payload;
  }
}

const updateProgress = (commit, mutation, id, progressEvent) => {
  if (progressEvent.lengthComputable) {
    commit(mutation, {
      id,
      value: (progressEvent.loaded / progressEvent.total) * 100
    });
  }
}

export const createModule = (options) => {
  const SERVER_PREFIX = (options.mode == null ? process.env.NODE_ENV : options.mode) === 'production' ? '' : '/server/';
  let m = {
    namespaced: true,
    state: {
      items: [],
      upload_progress: {},
      download_progress: {}
    },
    getters: {
      items (state) {
        return state.items;
      },
      getItemById: (state) => (id) => {
        return getItemById(state.items, id);
      },
      filterItems: (state) => (filter) => {
        return state.items.filter(filter);
      },
      getAttachmentUrl: (state) => (id, attachmentName) => {
        const item = getItemById(state.items, id);
        if (item && _.includes(item.attachment_names, attachmentName)) {
          return SERVER_PREFIX + options.entityPlural + '/' + id + '/attachments/' + attachmentName;
        } else {
          return null;
        }
      },
      getUploadProgressById: (state) => (id) => {
        if (id && _.has(state.upload_progress, id)) {
          return state.upload_progress[id];
        }
        return null;
      },
      getDownloadProgressById: (state) => (id) => {
        if (id && _.has(state.download_progress, id)) {
          return state.download_progress[id];
        }
        return null;
      },
      newId: (state) => {
        return uuidv4;
      }
    },
    mutations: {
      setItems (state, items) {
        state.items = items;
      },
      addItem (state, item) {
        if (!item.id) {
          Vue.set(item, 'id', uuidv4());
        }
        state.items.push(item);
      },
      replaceItem (state, item) {
        var itemIdx = getItemIdxById(state.items, item.id);
        if (!_.isNil(itemIdx)) {
          state.items.splice(itemIdx, 1, item);
        } else {
          state.items.push(item);
        }
      },
      replaceItems (state, items) {
        state.items = _.map(state.items, (item) => {
          const newItem = _.find(items, newItem => newItem.id === item.id);
          return newItem || item;
        });
      },
      removeItem(state, itemId) {
        var itemIdx = getItemIdxById(state.items, itemId);
        if (!_.isNil(itemIdx)) {
          state.items.splice(itemIdx, 1);
        }
      },
      setUploadProgress (state, payload) {
        Vue.set(state.upload_progress, payload.id, payload.value);
      },
      unsetUploadProgress (state, id) {
        if (_.has(state.upload_progress, id)) {
          Vue.delete(state.upload_progress, id);
        }
      },
      setDownloadProgress (state, payload) {
        Vue.set(state.download_progress, payload.id, payload.value);
      },
      unsetDownloadProgress (state, id) {
        if (_.has(state.download_progress, id)) {
          Vue.delete(state.download_progress, id);
        }
      }
    },
    actions: {
      createItem ({commit}, item) {
        if (!item.id) {
          Vue.set(item, 'id', uuidv4());
        }

        commit('startLoading', options.entityPlural + '.create', {root: true});
        let promise = axios.post(SERVER_PREFIX + options.entityPlural + '/', createPayload(item, options.entitySingular), {
          onUploadProgress (progressEvent) {
            updateProgress(commit, 'setUploadProgress', item.id, progressEvent);
          }
        });
        promise.then((resp) => {
          commit('endLoading', options.entityPlural + '.create', {root: true});
          commit('addItem', resp.data[options.entitySingular]);
        });
        promise.catch((error) => {
          commit('endLoading', options.entityPlural + '.create', {root: true});
          console.log(error);
        });
        return promise;
      },
      updateItem ({commit}, item) {
        commit('startLoading', options.entityPlural + '.update', {root: true});
        let promise = axios.put(SERVER_PREFIX + options.entityPlural + '/' + item.id, createPayload(item, options.entitySingular, false), {
          onUploadProgress (progressEvent) {
            updateProgress(commit, 'setUploadProgress', item.id, progressEvent);
          }
        });
        promise.then(resp => {
          commit('endLoading', options.entityPlural + '.update', {root: true});
          commit('replaceItem', resp.data[options.entitySingular]);
        });
        promise.catch(error => {
          commit('endLoading', options.entityPlural + '.update', {root: true});
          console.log(error);
        });
        return promise;
      },
      updateItems ({commit}, items) {
        commit('startLoading', options.entityPlural + '.update', {root: true});
        const payload = {}
        payload[options.entityPlural] = items.map(item => normalizeItem(item, false));
        let promise = axios.put(SERVER_PREFIX + options.entityPlural + '/', payload);
        promise.then(resp => {
          commit('endLoading', options.entityPlural + '.update', {root: true});
          commit('replaceItems', resp.data[options.entityPlural]);
        });
        promise.catch(error => {
          commit('endLoading', options.entityPlural + '.update', {root: true});
          console.log(error);
        });
        return promise;
      },
      fetchItems ({commit}) {
        commit('startLoading', options.entityPlural + '.read', {root: true});
        let promise = axios.get(SERVER_PREFIX + options.entityPlural + '/', {
          onDownloadProgress (progressEvent) {
            updateProgress(commit, 'setDownloadProgress', 'items', progressEvent);
          }
        });
        promise.then((response) => {
          commit('endLoading', options.entityPlural + '.read', {root: true});
          commit('setItems', response.data[options.entityPlural] || []);
        });
        promise.catch(error => {
          commit('endLoading', options.entityPlural + '.read', {root: true});
          console.log(error);
        });
        return promise;
      },
      fetchItemById ({commit}, itemId) {
        commit('startLoading', options.entityPlural + '.read', {root: true});
        let promise = axios.get(SERVER_PREFIX + options.entityPlural + '/' + itemId, {
          onDownloadProgress (progressEvent) {
            updateProgress(commit, 'setDownloadProgress', itemId, progressEvent);
          }
        });
        promise.then((response) => {
          commit('endLoading', options.entityPlural + '.read', {root: true});
          commit('replaceItem', response.data[options.entitySingular]);
        });
        promise.catch(error => {
          commit('endLoading', options.entityPlural + '.read', {root: true});
          console.log(error);
        });
        return promise;
      },
      deleteItem ({commit}, itemId) {
        commit('startLoading', options.entityPlural + '.remove', {root: true});
        let promise = axios.delete(SERVER_PREFIX + options.entityPlural + '/' + itemId);
        promise.then((response) => {
          commit('endLoading', options.entityPlural + '.remove', {root: true});
          if (response.data.ok) {
            commit('removeItem', itemId);
          }
        });
        promise.catch(error => {
          commit('endLoading', options.entityPlural + '.remove', {root: true});
          console.log(error);
        });
        return promise;
      },
      cloneItem ({commit}, itemId) {
        commit('startLoading', options.entityPlural + '.create', {root: true});
        let promise = axios.post(SERVER_PREFIX + options.entityPlural + '/' + itemId + '/clone', {id: uuidv4()});
        promise.then((resp) => {
          commit('endLoading', options.entityPlural + '.create', {root: true});
          commit('addItem', resp.data[options.entitySingular]);
        });
        promise.catch((error) => {
          commit('endLoading', options.entityPlural + '.create', {root: true});
          console.log(error);
        });
        return promise;
      }
    }
  }

  let extendableAttributes = ['getters', 'mutations', 'actions', 'state'];
  extendableAttributes.forEach((attrKey) => {
    if (options[attrKey]) {
      _.assignIn(m[attrKey], options[attrKey]);
    }
  });

  return m;
}
